/*
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 *
 */

using System;
using System.Collections.Generic ;
using System.Xml;
using System.IO;
using org.apache.qpid.console;
using org.apache.qpid.transport.util ;
using log4net;
using log4net.Config;

namespace org.apache.qpid.console
{
	
	
	public class Test
	{
		
		public Test()
		{
		}
		
		
	public static string CommonAttributes(SchemaVariable var) {
    	string returnString = "" ;
	    if (var.Description != null){
	       returnString = returnString + String.Format(" desc='{0}'", var.Description) ;  
	    }       
	    
	    if (var.RefPackage != null){
	       returnString = returnString + String.Format(" refPackage='{0}'", var.RefPackage) ;  
	    }  	    
	    	    
	    if (var.RefClass != null){
	       returnString = returnString + String.Format(" refClass='{0}'", var.RefClass) ;  
	    }  	
	    
	    if (var.Unit != null){
	       returnString = returnString + String.Format(" unit='{0}'", var.Unit) ;  
	    }  		    
	    
	    if (var.Min != null){
	       returnString = returnString + String.Format(" min='{0}'", var.Min) ;  
	    }	       
	    if (var.Max != null){
	       returnString = returnString + String.Format(" max='{0}'", var.Max) ;  
	    }	       
	    if (var.MaxLength != null){
	       returnString = returnString + String.Format(" maxLength='{0}'", var.MaxLength) ;  
	    }
	    
	    return returnString ;
      }

	public static void DumpXML(Session sess, string packageName) {
		System.Console.WriteLine("<schema package='{0}'>", packageName) ;
		foreach (ClassKey key in sess.GetClasses(packageName)) {
			System.Console.WriteLine("\t<!-- {0} -->", key.GetKeyString()) ;
			SchemaClass schema = sess.GetSchema(key) ;
			if (schema.Kind == 1) {
				if (schema.HasSuperType()) {
					System.Console.WriteLine("\t<class name='{0}' extends='{1}'>", key.ClassName, schema.SuperType) ;
				} else {
					System.Console.WriteLine("\t<class name='{0}'>", key.ClassName) ;
				}
				foreach (SchemaProperty prop in schema.Properties) {
					object[] attributes = new object[5] ;
					attributes[0] = prop.Name ;
					attributes[1] = Util.TypeName(prop.Type) ;
					attributes[2] = Util.AccessName(prop.Access) ;
					attributes[3] = prop.Optional ;
					attributes[4] = Test.CommonAttributes(prop);
					System.Console.WriteLine("\t\t<property name='{0}' type='{1}' access='{2}' optional='{3}'{4}/>", attributes) ;
				}
				foreach (SchemaMethod meth in schema.Methods) {
					System.Console.WriteLine("\t\t<method name='{0}'/>", meth.Name) ;	
					foreach (SchemaArgument arg in meth.Arguments) {
						object[] attributes = new object[4] ;					
						attributes[0] = arg.Name ;
						attributes[1] = arg.Direction ;
						attributes[2] = Util.TypeName(arg.Type) ;
						attributes[3] = Test.CommonAttributes(arg);					
						System.Console.WriteLine("\t\t\t<arg name='{0}' dir='{1}' type='{2}'{3}/>", attributes) ;
					}
					System.Console.WriteLine("\t\t</method> ") ;
				}
				System.Console.WriteLine("\t</class> ") ;				
			} else {
				if (schema.HasSuperType()) {
					System.Console.WriteLine("\t<event name='{0}' extends='{1}'>", key.ClassName, schema.SuperType) ;
				} else {
					System.Console.WriteLine("\t<event name='{0}'>", key.ClassName) ;
				}				
				foreach (SchemaArgument arg in schema.Arguments) {
					object[] attributes = new object[4] ;					
					attributes[0] = arg.Name ;
					attributes[1] = Util.TypeName(arg.Type) ;
					attributes[2] = Test.CommonAttributes(arg);					
					System.Console.WriteLine("\t\t\t<arg name='{0}' type='{1}'{2}/>", attributes) ;
				}
				System.Console.WriteLine("\t</event> ") ;
			}
		}
		System.Console.WriteLine("</schema> ") ;		
	}
		public static void Main(string[] args) {
			XmlConfigurator.Configure(new FileInfo("./log.xml"));
			ILog log = LogManager.GetLogger(typeof(Test)) ;			
			log.Debug("Starting") ;
			Session session = new Session() ;
			session.Console = new AbstractConsole() ;
			session.AddBroker("amqp://localhost") ;		
			
			Test.DumpXML(session, "com.redhat.vdc") ;
			Test.DumpXML(session, "com.redhat.vdc.action") ;		
			Test.DumpXML(session, "com.redhat.vdc.queries") ;	
			
			// Find an Object
			Dictionary<string, object> options = new Dictionary<string, object>() ;
			options.Add("_class", "Muppet") ;
			options.Add("_package", "org.apache.test") ;
			QMFObject obj = session.GetObjects(options)[0] ;
			System.Console.WriteLine(obj) ;	
			
			// Basic Method invocation
			MethodResult result = obj.InvokeMethod("doSomething", "Some String") ;
			System.Console.WriteLine(result) ;
			result = obj.InvokeMethod("returnSomething") ;
			System.Console.WriteLine(result) ;			
			
			// Return a Complex Object
			Dictionary<string, object> dict = new Dictionary<string, object>() ;
			dict.Add("e", 32321) ;
			result = obj.InvokeMethod("gimmieCrumpet", "asdf", 3l, 3.14f, dict) ;
			System.Console.WriteLine(result) ;			
			QMFObject crumpet = (QMFObject) result.GetReturnValue("result") ;
			List<object> ingredients = (List<object>) crumpet.GetProperty("ingredients") ;
			foreach (Object ing in ingredients) {
				System.Console.WriteLine(ing) ;			
			}
			
			// Send a Complex Object
			result = obj.InvokeMethod("takeCrumpet", crumpet) ;
			
			// Retrieve a List of Objects
			result = obj.InvokeMethod("gimmeLotsOfCrumpets") ;
			System.Console.WriteLine(result) ;		
			
			// Failure
			result = obj.InvokeMethod("divideByZero") ;
			System.Console.WriteLine(result) ;				
			
			
			//crumpet = session.MakeObject(crumpet.Schema.Key) ;
			//System.Console.WriteLine(crumpet) ;
			//esult = obj.InvokeMethod("takeCrumpet", crumpet) ;
			//System.Console.WriteLine(result) ;	
									
			/* 
			log.Debug("Waiting...") ;
			System.Threading.Thread.Sleep(1500) ;
			log.Debug("Done") ;*/
			session.Close() ;
		}
	}
}
