/*
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 *
 */

using System;
using System.Collections.Generic ;
using System.Reflection ;
using System.Xml;
using System.IO;
using org.apache.qpid.console;
using org.apache.qpid.transport.util ;
using log4net;
using log4net.Config;

namespace org.apache.qpid.console
{
	
	public class VdcConsole : AbstractConsole {
	
		public VdcConsole() : base() {
		}
		
		public override void EventRecieved(Broker broker, QMFEvent anEvent) {
			if ((anEvent.ClassKey.PackageName.Equals("com.redhat.vdc")) &&
				(anEvent.ClassKey.ClassName.Equals("QueryDataChangedEvent"))) {
	            System.Console.WriteLine( "*******************************************");
				System.Console.WriteLine(anEvent.GetArgument("sessionId")) ;
				System.Console.WriteLine(anEvent.GetArgument("queryId")) ;
				System.Console.WriteLine(anEvent.GetArgument("updatedData")) ;	
			}
		}
	}
	
	public class Frontend {
		public Session Session ;
		protected VdcConsole Console ;
		protected QMFObject Backend ;
		protected int SessionID = 33 ;
		
		public Frontend() {
			Console = new VdcConsole() ;
			Session = new Session() ;
			Session.Console = Console ;
			Session.RecieveEvents = true ;
			Session.AddBroker("amqp://localhost") ;
			Dictionary<string,object> args = new Dictionary<string, object>() ;
			args.Add("_class", "Backend") ;
			Backend = Session.GetObjects(args)[0] ;
		}
		
		public QMFObject RunAction(string actionType, VDCActionParameters parameters) {
			parameters.SetCallback("Garbage") ;
			MethodResult result = Backend.InvokeMethod("runAction", actionType, parameters, "" + SessionID) ;
			return (QMFObject) result.GetReturnValue("result") ;
		}
		
		public QMFObject RunQuery(string queryType, QMFObject parameters) {
			parameters.SetProperty("callback", "Garbage") ;
			MethodResult result = Backend.InvokeMethod("runQuery", queryType, parameters, "" + SessionID) ;
			return (QMFObject) result.GetReturnValue("result") ;
		}		
		
		public void Close() {
			Session.Close() ;
		}
		
		public QMFObject MakeObject(String key) {
			return Session.MakeObject(key) ;
		}
		
		public Object MakeObject(Type type) {
			Type[] types = {typeof(Session)} ;
			Object[] args = {Session} ;
			ConstructorInfo ctor = type.GetConstructor(types) ;
			return ctor.Invoke(args) ;
		}
	}
	
	
	// Unmanaged Object
	public class VDCActionParameters : QMFObject 
	{
	
		public static ClassKey CurrentClassKey() {
			return new ClassKey("com.redhat.vdc.action:VdcActionParameters(728a77a6-267f9a35-7e93a8fa-00000000)") ;
		}
		
		public VDCActionParameters(Session session) : base(session, session.GetSchema(VDCActionParameters.CurrentClassKey()), true, true, false) 
		{
		}
		
		public void SetCallback(string aValue) {
			this.SetProperty("callback", aValue) ;
		}
		
		public string GetCallback() {
			return (String) this.GetProperty("callback") ;
		}		
	}
	

	public class VDCQueryParameters : QMFObject 
	{
	
		public static ClassKey CurrentClassKey() {
			return new ClassKey("com.redhat.vdc.queries:VdcQueryParameters(4b100bf6-e89e90bd-7e93a8fa-00000000)") ;
		}
		
		public VDCQueryParameters(Session session) : this(session, session.GetSchema(VDCQueryParameters.CurrentClassKey()), true, true, false) 
		{
		}
		
		public VDCQueryParameters(Session session, SchemaClass schema, bool hasProperties, bool hasStats , bool isManaged):
				base(session, schema, hasProperties, hasStats, isManaged) {
		}
		
		public void SetCallback(string aValue) {
			this.SetProperty("callback", aValue) ;
		}
		
		public string GetCallback() {
			return (String) this.GetProperty("callback") ;
		}	
		
		public void SetQueryID(string aValue) {
			this.SetProperty("queryID", aValue) ;
		}
		
		public string GetQueryID() {
			return (String) this.GetProperty("queryID") ;
		}					
	}
		
	
	
	public class TestFrontend
	{
		
		public TestFrontend()
		{		
		}
		
		public static void Main(string[] args) {
			Frontend front = new Frontend() ;
			VDCActionParameters obj = (VDCActionParameters) front.MakeObject(typeof(VDCActionParameters)) ;
			//QMFObject obj = front.MakeObject("com.redhat.vdc.action:VdcActionParameters(728a77a6-267f9a35-7e93a8fa-00000000)") ;
			//obj.Properties.Add("groupId", 749) ;
			QMFObject returnObj = front.RunAction("PingBackend", obj) ;
			System.Console.WriteLine(returnObj) ;
			
			//obj = front.MakeObject("com.redhat.vdc.queries:VdsGroupQueryParameters(4b100bf6-dfec4f58-ffa7dfda-00000000)") ;
			//obj.SetProperty("groupId", 749) ;
			//returnObj = front.RunQuery("GetVdsGroupById", obj) ;
			//System.Console.WriteLine(returnObj) ;			
			
			front.Close() ;
		}		
		
	}
}
